#ifndef _MALLOC_INCLUDED_
#define _MALLOC_INCLUDED_     /* definujeme, ze je inkludovan malloc.h */
/*
 *  malloc.h
 *  zde jsou funkce na jednoduche alokovani a dealokovani pameti
 *  [prevzato z FreeDosu, uz presne nevim odkud >:) ] 
 *  typy:
 *    block         - hlavicka o alokovane pameti  
 *  funkce: 
 *    memcpy()      - nastavi cast pameti 
 *    malloc()      - alokuje cast pameti  
 *    free()        - dealokuje cast pameti  
 */
#include <_null.h>
#include <mem.h>
#include <types.h>

unsigned int __brklvl = 0x8000;      /* 
                                      *  misto v pameti, odkud se bude alokovat,
                                      *  idealni je dat toto cislo na hodnotu
                                      *  (velikost_aplikace+1)
                                      */
#define sbrk(x) ((void *)__brklvl)
#define brk(newbrk) \
	(((char *)(newbrk) > (char *)(&length) - 0x200) ? \
		-1 : \
		(__brklvl = (unsigned)(newbrk), 0))

#define BUSY	    (sizeof(size_t)-1)	/* jestli je pametovy blok pouzivany, vrati 1 */
#define isbusy(x)   ((x->length)&BUSY)


/******************************************************************************/
/****************************   DEFINICE TYPU   *******************************/
/******************************************************************************/
typedef union {
  size_t length;
  char data[1];
} block;

/*
 *  malloc()
 *  alokuje <length> bajtu a vrati pointer
 */  
void *malloc(size_t length) {
  static	block *alloc_bottom = NULL;
  block	*akt, *next;
  block	*ltop = sbrk(0);

  if (alloc_bottom == NULL)
    alloc_bottom = ltop;

  length = (length + sizeof(size_t) + BUSY) & ~BUSY;

  akt = alloc_bottom;
  for (;;) {
    if (akt == ltop) {                    /* konec inicializovane pameti */
      next = (block *)&akt->data[length];
      if (next < akt || brk(next) == -1)
        return NULL;
      break;
    }
    
    next = (block *)(&akt->data[akt->length & ~BUSY]);
    
    if (isbusy(akt)) {
      akt = next;                         /* dalsi blok	 */
    } else if (next == ltop || isbusy(next)) {
      size_t size = akt->length;
      if (size >= length) {               /* zkusit rozdelit */
        if (size > length) {              /* rozdeleni */ 
          ((block *)&akt->data[length])->length = size - length;
        }
        break;
      }
      akt = next;                         /* dalsi blok	 */
    }
    else {                                /* spojit dva bloky */
      akt->length += next->length;
    }
  }
  akt->length = length | BUSY;
  
  return &akt->data[sizeof(size_t)];
}

/*
 *  free()
 *  dealokuje (m)alokovanou cast
 */
void free(void *ptr)
{
  if (ptr)
    ((char *) ptr)[-sizeof(size_t)] &= ~BUSY;
}

/* 
 *  realloc()
 *  pripoji cast pameti k jiz alokovane pameti 
 */
void* realloc(void *ptr,size_t newlength)
{
  size_t oldlength = ((size_t*)ptr)[-1] & ~BUSY;
  void *newptr;

  newptr = malloc(newlength);

  if (newptr == NULL) {
    if (newlength <= oldlength)
      return ptr;
  } else {
    memcpy(newptr,ptr,oldlength);
  }

  free(ptr);
  return newptr;
}

#endif
