#ifndef _STRING_INCLUDED_
#define _STRING_INCLUDED_     /* definujeme, ze je inkludovan string.h */
/*
 *  string.h
 *  ruzne prace pro praci se stringy
 *  kopirovani, porovnavani, presouvani...  
 *  funkce:
 *    putchar()     - vytiskne znak
 *    strcat()      - spoji dva stringy
 *    strcmp()      - porovna dva stringy 
 *    strcpy()      - natavi string
 *    strlen()      - vrati delku stringu
 *    toupper()     - zvetsi pismeno (a-z) z maleho na velke
 *    tolower()     - zmensi pismeno (A-Z) z velkeho na male  
 *    upStr()       - zvetsi pismena ve stringu z malych na velke
 *    downStr()     - zmensi pismena ve stringu z velkych na male
 *    puts()        - vytiskne string (neformatovane)
 *    pos()         - urci pozici prvniho znaku ve string
 *    getchar()     - sejme znak z klavesnice, vrati jako [int]
 *    gets()        - sejme string z klavesnice
 *    itoa()        - prevede integer na string
 */  


#include <asm.h>  /* putch() */

/*
 *  putchar()
 *  vytiskne ASCII znak, pokud je vstup 0x08 (backspace), umaze jeden znak
 */ 
void putchar(char s) {
  if(s == '\b') {     /* backspace */
    putch('\b');
    putch(' ');
    putch('\b');
  } else {
    putch(s);
  }  
}

/*
 *  strcat()
 *  zkopiruje string pSrc na konec stringu pDst,
 *  spoji string pDst a pSrc, vrati v pDst... 
 */ 
char *strcat(char * pDst, const char *pSrc) {
    char *r = pDst;

    while (*pDst != '\0')
        pDst++;

    while ((*pDst++ = *pSrc++) != '\0')
        continue;

    return r;
}

/*
 *  strcmp()
 *  porovna string d se stringem s
 *  pokud jsou stringy shodne, vrati 0  
 */ 
int strcmp(char *d, char *s) {
  while (*s != '\0' && *d != '\0') {
    if (*d == *s)
      ++s, ++d;
    else
      return *d - *s;
  }
  return *d - *s;
}

/*
 *  strcpy()
 *  zkopiruje src do dest zakonceny '\0'
 */ 
char *strcpy(char * dest,const char *src) {
char *tmp = dest;
  while ((*dest++ = *src++) != '\0');
    return tmp;
}

/*
 *  strlen()
 *  vstup: ukazatel na zacatek stringu zakonceneho '\0'
 *  vystup: delka stringu (pocet znaku pred nulovym bajtem)  
 */ 
int strlen(const char *str) {
  static int i = 0;
  for(i = 0; *str != '\0'; str++) {
    i++;
  }
  return i;
}

/*
 *  toupper()
 *  zvetsi pismeno (a-z) z maleho na velke
 */ 
char toupper(char c) {
  if((c >= 'a') && (c <= 'z')) {
    c -= ' ';
  }
  return c;
}

/*
 *  tolower()
 *  zmensi pismeno (A-Z) z velkeho na male
 */ 
char tolower(char c) {
  if((c >= 'A') && (c <= 'Z')) {
    c += ' ';
  }
  return c;
}

/*
 *  upStr()
 *  prevede znaky na velka pismena ve stringu s1 do stringu s2 
 */ 
char *upStr(char *s1, char *s2) {
  int i = 0;
  for(;*s1!='\0';s1++,i++,s2++) {
    *s2 = toupper(*s1);
  }
  *s2 = '\0';
  s2 -= i;
  return s2;
}

/*
 *  downStr()
 *  prevede znaky na mala pismena ve stringu s1 do stringu s2 
 */ 
char *downStr(char *s1, char *s2) {
  int i = 0;
  for(;*s1!='\0';s1++,i++,s2++) {
    *s2 = tolower(*s1);
  }
  *s2 = '\0';
  s2 -= i;
  return s2;
}

/*
 *  puts()
 *  vstup: ukazatel na string, ktery se bude tisknout na vystup 
 *  vystup: void
 */
void puts(const char *str) {
  while(*str)
    putchar(*str++);
}

/*
 *  pos()
 *  urci pozici prvniho znaku <c> ve stringu <s> 
 */ 
int pos(char *s, char c) {
  int i=0;
  int l=strlen(s);
  for(;*s!=c;s++) {
    i++;
    if(i>=l) {
      i=-1;
      break;
    }
  }
  return ++i;
}

/*
 *  getchar()
 *  sejme znak z klavesnice a vrati ho jako [int] 
 */ 
int getchar() {
 char character = 0;
 character = getch();

 return (int) character;
}

/*
 *  gets() 
 *  sejme string z klavesnice o maximalni delce <size>, kazdy znak vytiskne na STDOUT
 */ 
char *gets(char *cs, int size) {
  unsigned int counter = 0;
  unsigned char c = '\0';
  char *s;

  s = cs;
  while (1) {
  if(counter >= size) break;
  c = getch();
  if(c == '\r') break;
  if(c == '\n') break;
  if(c == '\0') break;
  if(c == '\t') c = ' ';

    if(counter == 0 && c == '\b') {} else {
      putchar(c);
    }
    if( c == '\b' ) {
      if (counter > 0)
      counter--;
    }
    else {
      *(s+(counter++)) = c;
    }
  }
  puts("\r\n");
  *(s+(counter++)) = '\0';
  return s;
}

/* 
 *  itoa()
 *  prevede integer na string,
 *  ktery vrati ve vystupu (ukazatel na nej).
 *  _[PREVZATO A UPRAVENO Z http://en.wikipedia.org/wiki/Itoa]_  
 */
char *itoa(int n) {
  int i, sign, c, k, j;
  static char *s = "";

  if ((sign = n) < 0) {
      n = -n;
  }      
  i = 0;
  do {
      s[i++] = n % 10 + '0';
  } while ((n /= 10) > 0);
  if (sign < 0) {
      s[i++] = '-';
  }
  s[i] = '\0';
  j = (strlen(s)-1);
  for (k = 0; k<j; k++, j--) {
    c = s[k];
    s[k] = s[j];
    s[j] = c;
  }
  return s;
} 

#endif
