#ifndef _VIDEO_INCLUDED_
#define _VIDEO_INCLUDED_     /* definujeme, ze je inkludovan video.h */
/*
 *  video.h
 *  ruzne fce pro praci s monitorem a textovym kurzorem
 *    
 *  funkce:
 *    gotoxy()      - posune textovy kurzor na radek x, sloupec y
 *    getxy()       - zjisti v jakem radku a sloupci se nachazi textovy kurzor
 *    vidmode()     - nastavi danne video mode
 *    textcolor()   - "alokuje" znaky v konzoli barvou 
 *    prowdown()    - prida x radku ze spoda, se zadanou barvou
 *    prowup()      - prida x radku ze zhora, se zadanou barvou
 *    rowdown()     - prida x radku ze spoda, s implictni barvou
 *    rowup()       - prida x radku ze zhora, s implictni barvou
 *    clrscr()      - vymaze obrazovku, presune kurzor na 0, 0 
 *    scrchr()      - vrati znak, ktery je na aktualni pozici kurzoru
 *    scrchrxy()    - vrati znak, ktery je na pozici x,y
 *    charto()      - napise znak <ch> na pozici <x, y> 
 */

#include <global.h>
#include <string.h>   /* strlen() */
#include <types.h>    /* byte     */ 

/******************************************************************************/
/****************************   DEFINICE TYPU   *******************************/
/******************************************************************************/

#define BLACK             0   /* dark colors */
#define BLUE              1
#define GREEN             2
#define CYAN              3
#define RED               4
#define MAGENTA           5
#define BROWN             6
#define LIGHTGRAY         7
#define DARKGRAY          8   /* light colors */
#define LIGHTBLUE         9
#define LIGHTGREEN        10
#define LIGHTCYAN         11
#define LIGHTRED          12
#define LIGHTMAGENTA      13
#define YELLOW            14
#define WHITE             15

/******************************************************************************/
/********************************   FUNKCE   **********************************/
/******************************************************************************/

/*
 *  gotoxy()
 *  posune textovy kurzor na radek x, sloupec y
 */
void gotoxy(char x, char y) {
  _AH = 0x02;    
  _BH = 0;    
  _DH = x;
  _DL = y;   
  __int__(0x10);
}

/*
 *  getxy()
 *  zjisti v jakem radku (x) a sloupci (y) se nachazi textovy kurzor
 */
void getxy(int *x, int *y) {
  _AH = 0x03;    
  _BH = 0;
  __int__(0x10);
  
  *x = _DH;
  *y = _DL;
}
 
/*
 *  vidmode()
 *  nastavi danne video mode pres int 10h,
 *  nejvhodnejsi mod v pomeru text/grafika je vidmode(0x10) 
 */ 
void vidmode(int mode) {
  _AH = 0x00;
  _AL = mode;
  __int__(0x10);
}

/*
 *  textcolor()
 *  zmeni barvu pisma na <col>  
 */
void textcolor(byte col) {
  TEXT_COL = col;
}

/*
 *  prowdown()
 *  prida (int rows) prazdnych radku s barvou <mode> na zacatek obrazovky,
 *  maze radky odspoda. 0 maze obrazovku, nepresouva ukazatel 
 */ 
void prowdown(int rows, int mode) {
  _AH = 0x07;

  _CH = 0;
  _CL = 0;
  
  _DH = 200;
  _DL = 200;
  
  _AL = rows;
  _BH = mode;
  
  __int__(0x10);
}

/*
 *  prowup()
 *  prida (int rows) prazdnych radku s barvou <mode> na konec obrazovky,
 *  maze radky zvrcha. 0 maze obrazovku, nepresouva ukazatel 
 */ 
void prowup(int rows, int mode) {
  _AH = 0x06;

  _CH = 0;
  _CL = 0;
  
  _DH = 200;
  _DL = 200;
  
  _AL = rows;
  _BH = mode;
  
  __int__(0x10);
}

/*
 *  rowdown()
 *  prida (int rows) prazdnych radku na zacatek obrazovky,
 *  maze radky odspoda. 0 maze obrazovku, nepresouva ukazatel 
 */ 
void rowdown(int rows) {
  prowdown(rows, 0x07);
}

/*
 *  rowup()
 *  prida (int rows) prazdnych radku na konec obrazovky,
 *  maze radky zvrcha. 0 maze obrazovku, nepresouva ukazatel 
 */ 
void rowup(int rows) {
  prowup(rows, 0x07);
}

/*
 *  clrscr()
 *  vymaze obrazovku vlozenim radku, nakonec presune
 *  kurzor na pozici 0, 0  
 */ 
void clrscr() {
  rowdown(0);
  gotoxy(0,0);
}

/*
 *  scrchr()
 *  vrati znak, ktery je na aktualni pozici kurzoru
 */ 
char scrchr() {
  _AH = 0x08;
  _BL = 0;
  __int__(0x10);
  return _AL;
}

/*
 *  scrchrxy()
 *  vrati znak, ktery je na pozici x,y
 */
char scrchrxy(int x, int y) {
  int ox = 0, oy = 0;
  char znak;
  getxy(&ox, &oy);
  gotoxy(x,y);
  znak = scrchr();
  gotoxy(ox, oy);
  return znak;
}

/*
 *  charto()
 *  napise znak <ch> na pozici <x, y>
 */
void charto(int x, int y, char ch) {
  int mx = 0, my = 0;
  getxy(&mx, &my);
  gotoxy(x,y);
  putchar(ch);
  gotoxy(mx,my);
}

#endif
